<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services;

use App\Jobs\RhawnSorderRefundJob;
use App\Repositories\Contracts\RhawnSoitemsRepository;
use App\Repositories\Contracts\RhawnSorderRefundRepository;
use App\Repositories\Contracts\RhawnSordersRepository;
use App\Repositories\Contracts\ToolsTaskRepository;
use App\Repositories\Criteria\RhawnSorderRefundCriteria;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\AdminUsers;
use App\Support\Traits\Helpers;
use Illuminate\Support\Facades\DB;


class RhawnOrdersService
{

    use Helpers;

    /**
     * 获取订单详情
     * @param $request
     */
    public function getSorderDetail($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');
        if(!isset($params['order_no']) || empty($params['order_no'])){
            throw new BadQueryStringException('订单编号错误',500);
        }

        $sOrder = app(RhawnSordersRepository::class)->getSorderFromOrderNo($params['order_no']);
        if(!$sOrder){
            throw new \LogicException('该订单编号不存在!',500);
        }

        //查询订单的详情
        $soItems = app(RhawnSoitemsRepository::class)->getSorderItemsDetailFromOrderId($sOrder->so_id);
        $soItems = $soItems->toArray();
        $returnItem = [];
        if($soItems){
            foreach($soItems as $item){
                $temp = [];
                $temp['b_cn_name'] = $item['b_cn_name'];
                $temp['p_code'] = $item['p_code'];
                $temp['c_cas'] = $item['c_cas'];
                $temp['p_cn_name'] = $item['p_cn_name'];
                $temp['si_price'] = $item['si_price'];
                $temp['si_discount'] = $item['si_discount'];
                $temp['si_p_tod'] = $item['si_p_tod'];
                $temp['si_num'] = $item['si_num'];
                $temp['si_amount'] = $item['si_amount'];
                $temp['si_vamount'] = $item['si_vamount'];
                $temp['si_note'] = $item['si_notes'];
                $temp['si_num'] = $item['si_num'];
                $temp['p_id'] = $item['p_id'];
                $temp['si_id'] = $item['si_id'];
                $temp['so_no'] = $sOrder->so_no;
                if($item['si_if_cancel'] == 0){
                    $returnItem[] = $temp;
                }
            }
        }

       return $returnItem;
    }

    /**
     * 获取销售订单库存批次
     * @param $request
     */
    public function getSordersStock($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $repository = app(RhawnSoitemsRepository::class);
        $result = $repository->getOrderItemsStockFromItemIds($params['order_item_id']);
        $returnItemStock = [];
        if($result && !empty($result)){
            foreach($result as $res){
               $temp = [];
               $temp['dpd_num'] = $res['dpd_num'];
               $temp['si_id'] = $res['si_id'];
               $temp['pstk_id'] = $res['pstk_id'];
               $temp['pstk_num'] = $res['pstk_num'];
               $temp['pstk_no'] = $res['pstk_no'];
               $returnItemStock[] = $temp;
            }
        }

        return $returnItemStock;
    }

    public function addSordersRefundTask($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        $sOrder = app(RhawnSordersRepository::class)->getSorderFromOrderNo($params['order_no']);
        if(!$sOrder){
            throw new \LogicException('该订单编号不存在!',500);
        }

        if(isset($params['task_id']) && !empty($params['task_id'])){
            if(!app(RhawnSorderRefundRepository::class)->find($params['task_id'])){
                throw new \LogicException('该任务不存在！',500);
            }
        }

        if($params['transfer_pre'] == 'true'){
            $params['transfer_pre'] = 1;
        }else{
            $params['transfer_pre'] = 0;
        }

        //先查询库存信息
        $repository = app(RhawnSoitemsRepository::class);
        $stockResult = $repository->getOrderItemsStockFromItemIds($params['item_id']);
        if(empty($stockResult) || !$stockResult){
            throw new \LogicException('订单明细id不存在！',500);
        }

        // 验证库存和发货数量
        if(!empty($params['refund_nums'])){
            $pstk_ids = $refundNums = [];
            foreach (explode(',',$params['refund_nums']) as $num) {
                $pstkArr = explode('|', $num);
                $refundNums[$pstkArr[0]] = $pstkArr[1];
                $pstk_ids[] = $pstkArr[0];
            }
            //分组查询发货数量
            $dbConnect = DB::connection('rhawn_mysql');
            $dpdSum = $dbConnect->table('dpdetail')
                ->whereIn('pstk_id', $pstk_ids)
                ->where('dpd_invoiced', '=', 0)
                ->groupBy('pstk_id')
                ->select(DB::raw("sum(dpd_num) as num"),)
                ->get();

            if (!$dpdSum || empty($dpdSum)) {
                throw new \LogicException('【' . implode(',', $pstk_ids) . '】对应的配货单数据不存在', 500);
            }
            if (count($dpdSum) < count($refundNums)) {
                throw new \LogicException('需要退货的配送单不存在或已开票', 500);
            }
            collect($dpdSum)->map(function ($dpd) use ($refundNums) {
                foreach ($refundNums as $num) {
                    if ($num > $dpd) {
                        throw new \LogicException('退货数量大于购买数量', 500);
                    }
                }
            });
        }

        return app(RhawnSorderRefundRepository::class)->addRefundTask($params);
    }

    /**
     * 删除销售订单退货任务
     * @param $request
     * @return mixed
     */
    public function delSorderRefundTask($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        if(isset($params['task_id']) && !empty($params['task_id'])){
            if(!app(RhawnSorderRefundRepository::class)->find($params['task_id'])){
                throw new \LogicException('该任务不存在！',500);
            }
        }

        return app(RhawnSorderRefundRepository::class)->delRefundTask($params);
    }

    /**
     * 获取订单退货任务
     * @param $request
     * @return array
     */
    public function getSordersRefundTask($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        if(!$params['page_size'] || $params['page_size'] == 0){
            $params['page_size'] = 10;
        }

        $repostiroy = app(RhawnSorderRefundRepository::class);
        $repostiroy->pushCriteria(new RhawnSorderRefundCriteria($request));
        $tasks = $repostiroy->customPaginate($params['page_size']);

        if($tasks){
            $allItems = collect($tasks['data'])->map(function($item){
                $contentArr= [];
                $execArr = json_decode($item['exec_content'],true);
                foreach($execArr as $key => $content){
                    switch($key){
                        case 'order_no' : $contentArr[] = '【销售订单号】：'.$content; break;
                        case 'refund_nums' :
                            if(!empty($content)){
                                $nums = explode(',',$content);
                                $subContentArr = [];
                                foreach($nums as $k => $num){
                                    $temp = explode('|',$num);
                                    $subContentArr[] = '<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;【批号id】：'.$temp[0];
                                    $subContentArr[] = '<br />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;【退货数量】：'.$temp[1];
                                }
                                $contentArr[] = '【现货退货数量】：'.implode("", $subContentArr); break;
                            }else{
                                $contentArr[] = '【现货退货数量】：0'; break;
                            }
                        case 'handle_fee' : $contentArr[] = '【手续费费用】：'.$content; break;
                        case 'item_id' : $contentArr[] = '【销售订单明细项】：'.$content; break;
                        case 'transfer_pre' :
                            $content = $content == '1' ? '是' : '否';
                            $contentArr[] = '【是否转预存】：'.$content; break;
                        case 'futures_num' :
                            $contentArr[] = '【期货退货数量】：'.$content; break;
                    }
                }

                $user = AdminUsers::find($item['create_by']);
                $item['create_by'] = $user->username;

                $item['content_string'] = implode('<br />',$contentArr);
                $item = array_merge($item,$execArr);
                return $item;
            });
            $tasks['data'] = $allItems->toArray();
            $tasks = $this->formatKeysfromArray($tasks);
        }

        return $tasks;
    }

    /**
     * 通过任务id获取销售订单任务详情
     * @param $request
     * @return array
     */
    public function getRhawnSorderDetailByTaskId($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        $task = app(RhawnSorderRefundRepository::class)->find($params['task_id']);
        if(!$task){
           throw new \LogicException('任务不存在!',500);
        }
        $task = $task->toArray();

        $content = json_decode($task['exec_content'],true);

        $pstkNums = [];
        if(!empty($content['refund_nums'])){
            foreach(explode(',',$content['refund_nums']) as $num){
                $numArr = explode('|',$num);
                $pstkNums[$numArr[0]] = $numArr[1];
            }
        }

        //获取销售订单item数据
        $result = app(RhawnSoitemsRepository::class)->getOrderItemsStockFromItemIds($content['item_id']);
        $returnItemStock = [];
        if($result && !empty($result)){
            foreach($result as $res){
                $temp = [];
                $temp['dpd_num'] = $res['dpd_num'];
                $temp['si_id'] = $res['si_id'];
                $temp['pstk_id'] = $res['pstk_id'];
                $temp['pstk_num'] = $res['pstk_num'];
                $temp['pstk_no'] = $res['pstk_no'];
                $temp['p_cn_name'] = $res['p_cn_name'];
                $temp['refund_nums'] = $pstkNums[$res['pstk_id']];
                $returnItemStock[] = $temp;
            }
        }
        return $returnItemStock;
    }

    /**
     * 手动执行脚本
     * @param $request
     */
    public function execSordersRefundTask($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        $orderRefund = app(RhawnSorderRefundRepository::class)->find($params['task_id']);

        $refundJob = (new RhawnSorderRefundJob($orderRefund->toArray()))->delay(100)->onQueue('slow');
        app('Illuminate\Contracts\Bus\Dispatcher')->dispatch($refundJob);
    }

    /**
     * 处理退货逻辑
     * @param $refundInfo
     */
    public function sOrderRefund($refundInfo,&$note)
    {
        $sOrder = app(RhawnSordersRepository::class)->getSorderItemsFromOrderNo($refundInfo->order_no);
        if(!$sOrder){
            throw new \LogicException('该订单号不存在!',500);
        }

        try{

            $dbConnect = DB::connection('rhawn_mysql');
            $dbConnect->beginTransaction();

            // 处理订单退货项
            foreach($sOrder->toArray() as $orderitem){
                if($refundInfo->item_id == $orderitem['si_id']) {

                    // 先处理源数据
                    $currentPrice = floatval($orderitem['si_discount']);
                    $currentAmount = floatval($orderitem['si_amount']);
                    // 退货剩余数量
                    $numArr = !empty($refundInfo->refund_nums) ? explode(',', $refundInfo->refund_nums) : '';
                    $futuresNum = $refundInfo->futures_num && $refundInfo->futures_num > 0 ? $refundInfo->futures_num : 0 ;


                    // 预判断退货数量
                    if (!empty($numArr)) {
                        $pstk_ids = [];
                        $refundNums = [];
                        foreach ($numArr as $num) {
                            $pstkArr = explode('|', $num);
                            $refundNums[$pstkArr[0]] = $pstkArr[1];
                            $pstk_ids[] = $pstkArr[0];
                        }
                        //分组查询发货数量
                        /*$dpdSum = $dbConnect->table('dpdetail')
                            ->whereIn('pstk_id', $pstk_ids)
                            ->where('dpd_invoiced', '=', 0)
                            ->groupBy('pstk_id')
                            ->select(DB::raw("sum(dpd_num) as num"),)
                            ->get();

                        if (!$dpdSum || empty($dpdSum)) {
                            throw new \LogicException('【' . implode(',', $pstk_ids) . '】对应的配货单数据不存在', 500);
                        }
                        if (count($dpdSum) < count($refundNums)) {
                            throw new \LogicException('需要退货的配送单不存在或已开票', 500);
                        }
                        collect($dpdSum)->map(function ($dpd) use ($refundNums) {
                            foreach ($refundNums as $num) {
                                if ($num > $dpd) {
                                    throw new \LogicException('退货数量大于购买数量', 500);
                                }
                            }
                        });*/
                    }

                    // 如果是期货，强行初始一个数组执行一次
                    if($futuresNum > 0 && empty($refundNums)){
                        $refundNums[0] = $futuresNum;
                    }

                    if (!empty($refundNums)) {

                        foreach ($refundNums as $pstk_id => $num) {

                            while (true) {

                                //修改item数据项
                                $service = app(RhawnSoitemsRepository::class);
                                $soItem = $service->findWhere(['si_id' => $orderitem['si_id'], 'si_if_cancel' => 0]);
                                $soItem = current($soItem->toArray());
                                if (empty($soItem) || count($soItem) == 0) {
                                    throw new \LogicException('购买项数据id:' . $orderitem['si_id'] . '数据不存在或已取消!', 500);
                                }

                                //如果有退货期货，检查退货期货数量是否大于当前期货数量
                                if ($futuresNum > 0) {
                                    if($pstk_id > 0){
                                        $dpdSum = $dbConnect->table('dpdetail')
                                            ->where('pstk_id', $pstk_id)
                                            ->where('si_id', $refundInfo->item_id)
                                            ->where('dpd_invoiced', '=', 0)
                                            ->groupBy('pstk_id')
                                            ->select(DB::raw("sum(dpd_num) as num"),)
                                            ->get()->first();
                                    }else{
                                        $dpdSum = $dbConnect->table('dpdetail')
                                            ->where('si_id',$refundInfo->item_id)
                                            ->where('dpd_invoiced', '=', 0)
                                            ->select(DB::raw("sum(dpd_num) as num"),)
                                            ->get()->first();
                                    }

                                    if(($soItem['si_num'] - $dpdSum->num) == 0){
                                        $futuresNum = 0;
                                        if($pstk_id == 0){
                                            throw new \LogicException('当前数据不存在期货,无法对期货进行退货', 500);
                                        }
                                    }

                                    if($futuresNum > 0) {
                                        if ($futuresNum > ($soItem['si_num'] - $dpdSum->num)) {
                                            throw new \LogicException('期货退货数量大于可退货数量', 500);
                                        }
                                    }
                                }

                                // 判断现货退货数量是否大于soitem的数量
                                if ($num > $soItem['si_num']) {
                                    throw new \LogicException('现货退货数量大于可退货数量', 500);
                                }

                                // 如果有期货数量优先处理
                                if ($futuresNum > 0) {
                                    $currentProcessNums = $futuresNum;
                                }else{
                                    $currentProcessNums = $num;
                                }

                                //更新原本数据
                                $updateArr = [];
                                $createArr = [];
                                $updatedItem = [];

                                //如果取消的是全部
                                if (($currentProcessNums - $orderitem['si_num']) == 0) {
                                    // 如果需要手续费
                                    if (isset($refundInfo->handle_fee) && $refundInfo->handle_fee > 0) {

                                        $createArr = $soItem;
                                        unset($createArr['si_id']);
                                        $createArr['si_num'] = $orderitem['si_num'];
                                        $createArr['si_assigned_num'] = $createArr['si_num'];
                                        $createArr['si_amount'] = bcsub($orderitem['si_amount'], $refundInfo->handle_fee, 2);
                                        $createArr['si_discount'] = bcdiv($createArr['si_amount'], $orderitem['si_num'], 2);

                                        $updateArr['si_num'] = $currentProcessNums;
                                        //$updateArr['si_assigned_num'] = $updateArr['si_num'];
                                        $updateArr['si_amount'] = $refundInfo->handle_fee;
                                        $updateArr['si_discount'] = bcdiv($updateArr['si_amount'], $currentProcessNums, 2);

                                    } else {
                                        $updatedItem['cancel_si_id'] = $orderitem['si_id'];
                                    }

                                } // 部分退货
                                else {

                                    if ($futuresNum > 0) {
                                        $updateArr['si_num'] = $orderitem['si_num'] - $futuresNum;
                                        //$updateArr['si_assigned_num'] = $orderitem['si_assigned_num'] - $futuresNum;
                                    } else {
                                        $updateArr['si_num'] = bcsub($orderitem['si_num'], $currentProcessNums); // 如果数量只有1，且退货数量相等
                                        $updateArr['si_assigned_num'] = $updateArr['si_num'];
                                    }

                                    $updateArr['si_amount'] = bcmul(
                                        $orderitem['si_discount'],
                                        $updateArr['si_num'],
                                        2
                                    );
                                    if (isset($refundInfo->handle_fee) && $refundInfo->handle_fee > 0) {
                                        $updateArr['si_amount'] = bcadd($updateArr['si_amount'], $refundInfo->handle_fee, 2);
                                    }
                                    $updateArr['si_discount'] = bcdiv($updateArr['si_amount'], $updateArr['si_num'], 2);

                                    // 新增
                                    $createArr = $soItem;
                                    unset($createArr['si_id']);
                                    $createArr['si_num'] = $currentProcessNums;

                                    if (isset($refundInfo->handle_fee) && floatval($refundInfo->handle_fee) > 0) {
                                        $createArr['si_amount'] = abs(bcsub($currentAmount, $updateArr['si_amount'], 2));
                                        $createArr['si_discount'] = bcdiv($createArr['si_amount'], $createArr['si_num'], 2);
                                    } // 没有手续费的情况下
                                    else {
                                        $createArr['si_discount'] = $soItem['si_discount'];
                                        $createArr['si_amount'] = bcmul($soItem['si_discount'], $currentProcessNums, 2);
                                    }
                                    $createArr['si_num'] = $currentProcessNums;
                                    if ($futuresNum > 0) {
                                        $createArr['si_assigned_num'] = 0;
                                    } else {
                                        $createArr['si_assigned_num'] = $createArr['si_num'];
                                    }

                                    if ($createArr['si_amount'] < 0) {
                                        throw new \LogicException('手续费异常,处理失败!', 500);
                                    }
                                }

                                if (!empty($updateArr)) {
                                    $update = $service->update($updateArr, $soItem['si_id']);
                                    $updatedItem['update'] = $update->toArray();
                                } else {
                                    $updatedItem['update'] = $orderitem;
                                }

                                //判断是否能除尽
                                if (!empty($updateArr)) {
                                    $amount = bcmul($updateArr['si_discount'], $updateArr['si_num'], 2);
                                    //if ($updateArr['si_amount'] != $amount) {
                                    if (bccomp($updateArr['si_amount'],$amount) != 0) {
                                        $note[] = 'si_id = ' . $soItem['si_id'] . '的数据有除不尽的情况,请手工处理';
                                    }
                                }

                                if (!empty($createArr)) {
                                    $newItem = $service->create($createArr);
                                    $updatedItem['create'] = $newItem->toArray();
                                    // 如果是部分退货
                                    if ($currentProcessNums > 0 || ($currentProcessNums == 0 && $refundInfo->handle_fee > 0)) {
                                        $updatedItem['cancel_si_id'] = $updatedItem['create']['si_id'];
                                    }
                                } else {
                                    $updatedItem['create'] = $soItem;
                                    $updatedItem['cancel_si_id'] = $updatedItem['create']['si_id'];
                                }

                                $updatedItem['total'] = $currentAmount;

                                // 如果没有期货数量,dpdetail只处理现货数量
                                if ($futuresNum == 0) {

                                    // 获取实际现货数量
                                    $dpdDetail = $dbConnect->table('dpdetail')
                                        ->where('si_id', $orderitem['si_id'])
                                        ->where('pstk_id', $pstk_id)
                                        ->get()->first();

                                    if ($dpdDetail) {
                                        // 更新当前记录
                                        $dpUpdateArr = [];
                                        $dpUpdateArr['dpd_num'] = $dpdDetail->dpd_num - $currentProcessNums;
                                        $dpUpdateArr['dpd_amount'] = bcmul($updatedItem['update']['si_discount'], $dpUpdateArr['dpd_num'], 2);
                                        // 出现全部退货
                                        if($dpUpdateArr['dpd_num'] == 0){
                                           if($refundInfo->handle_fee > 0){
                                               $dpUpdateArr['dpd_num'] = $dpdDetail->dpd_num;
                                               $dpUpdateArr['dpd_amount'] = bcmul($updatedItem['update']['si_discount'], $currentProcessNums, 2);
                                           } // 如果没有手续费，只修改对应的si_id
                                           else{
                                               $dpUpdateArr['si_id'] = $updatedItem['create']['si_id'];
                                               $dpUpdateArr['dpd_amount'] = $dpdDetail->dpd_amount;
                                               $dpUpdateArr['dpd_num'] = $dpdDetail->dpd_num;
                                           }
                                        } // 如果不是全部退货
                                        else{
                                            if($refundInfo->handle_fee > 0){
                                                $otherDpdDetail = $dbConnect->table('dpdetail')
                                                    ->where('si_id', $orderitem['si_id'])
                                                    ->whereNotIn('pstk_id', [$pstk_id])
                                                    ->get()->first();
                                                if($otherDpdDetail){
                                                    $otherUpdate = [];
                                                    $otherUpdate['dpd_amount'] = bcmul($updatedItem['update']['si_discount'], $otherDpdDetail->dpd_num, 2);
                                                    $dbConnect->table('dpdetail')
                                                        ->where('dpd_id', $otherDpdDetail->dpd_id)
                                                        ->update($otherUpdate);
                                                }
                                            }
                                        }
                                        $dbConnect->table('dpdetail')
                                            ->where('dpd_id', $dpdDetail->dpd_id)
                                            ->update($dpUpdateArr);
                                        // 如果是部分退货
                                        if (($currentProcessNums < $dpdDetail->dpd_num ) || ($currentProcessNums == $dpdDetail->dpd_num && $refundInfo->handle_fee > 0)) {
                                            //添加新的取消记录
                                            $dpCreateArr = $dpdDetail;
                                            unset($dpCreateArr->dpd_id);

                                            $dpCreateArr->dpd_num = $currentProcessNums;
                                            $dpCreateArr->dpd_amount = bcmul($updatedItem['create']['si_discount'], $currentProcessNums, 2);
                                            $dpCreateArr->si_id = $updatedItem['create']['si_id'];
                                            $dpId = $dbConnect->table('dpdetail')->insertGetId((array)$dpCreateArr);
                                        }
                                    }
                                }
                                //预存款和退款处理
                                $this->cancelAndTransPrePay($dbConnect, $updatedItem['cancel_si_id'], $refundInfo);

                                if ($futuresNum > 0) {
                                    $futuresNum = 0;
                                    $currentProcessNums = 0;
                                    if(empty($numArr)) break;
                                } else {
                                    break;
                                }
                            }
                        }
                    }
                }
            }

            $dbConnect->commit();

        }catch(\Exception $e){
            $dbConnect->rollBack();
            throw $e;
        }

    }

    /**
     * 取消并转预存
     * @param $db
     * @param $si_id
     */
    protected function cancelAndTransPrePay($db,$si_id,$refundInfo)
    {
        //获取订单明细项
        $sOrderItem = app(RhawnSoitemsRepository::class)->getSorderItemFromItemId($si_id);
        $sOrderItem = current($sOrderItem->toArray());
        if($sOrderItem['si_if_cancel'] == 1){
            throw new \LogicException('已经取消过了，请不要重复操作！',500);
        }

        $sOrderDetail = app(RhawnSordersRepository::class)->getSorderDetailFromOrderId($sOrderItem['so_id']);
        if(!$sOrderDetail){
            throw new \LogicException('数据不存在!',500);
        }
        $sOrderDetail['express'] = 0;
        foreach($sOrderDetail['items'] as $k=>$v){
            if($v->p_id == 0){
                $sOrderDetail['express'] = $v->si_amount;
                unset($sOrderDetail['items'][$k]);
            }
        }

        /*if($sOrderDetail['so_pay_status'] == 0){
            throw new \LogicException('未完成付款的订单不能取消转预存款！',500);
        }*/
        if($sOrderDetail['so_pay_status'] == 1){
            throw new \LogicException('部分付款的订单不能取消明细，请联系管理员！',500);
        }

        if($sOrderItem['pi_id'] > 0){
            //判断对应的采购订单明细是否已经取消
            $pOrderItems = $db->table('poitems')
                ->join('porders','poitems.po_id','porders.po_id')
                ->where('poitems.pi_id',$sOrderItem['pi_id'])
                ->first();
            if(!$pOrderItems){
                throw new \LogicException('查询采购订单明细 ['.$sOrderItem['pi_id'].'] 数据不存在!',500);
            }
            if($pOrderItems->pi_status != 3 && $refundInfo->cancel_porder == 1){
                throw new \LogicException('请先取消采购订单：'.$pOrderItems->po_no.'中的对应采购订单明细！');
            }
        }

        if($sOrderItem['si_assigned_num'] > 0){
            $dp_status = array();
            $invoiced_amount = 0;
            $dpdetailInfos = $db->table('dpdetail')
                ->join('dispatch','dpdetail.dp_id','dispatch.dp_id')
                ->where('dpdetail.si_id',$sOrderItem['si_id'])
                ->get();
            if(!$dpdetailInfos){
               throw new \LogicException('dpdetail ['.$sOrderItem['si_id'].'] 配货单数据不存在!',500);
            }
            foreach($dpdetailInfos as $k=>$v){
                $dp_status[] = $v->dp_status;
                $invoiced_amount += $v->dpd_invoiced;
            }
            if($invoiced_amount != 0){
                throw new \LogicException('已经开过发票，不能取消！',500);
            }
            $dp_status = array_unique($dp_status);
            if(in_array('0',$dp_status) || in_array('1',$dp_status)){
                throw new \LogicException('请物流部配合处理至已发货状态，再做取消操作！',500);
            }
        }

        // 具体处理退货并转预存
        $si_cancel_time = 0;

        $siAmountTotal = $db->table('soitems')
            ->where('so_id',$sOrderItem['so_id'])
            ->where('si_id','<>',$si_id)
            ->where('si_if_cancel',0)
            ->sum('si_amount');
        $siAmountTotalPid = $db->table('soitems')
            ->where('so_id',$sOrderItem['so_id'])
            ->where('si_id','<>',$si_id)
            ->where('si_if_cancel',0)
            ->where('p_id','>',0)
            ->sum('si_amount');

        $soItem = app(RhawnSoitemsRepository::class)->find($si_id);
        if($soItem){
            if($soItem['si_assigned_num']){
                $dp = $db->table('dpdetail')->where('si_id',$si_id)->get();
                $dp = $dp->toArray();
                foreach($dp as $k=>$v){
                    if($v->dp_id == 0 && $v->stk_id > 0){
                        //清除已配货记录
                        $stock = $db->table('stock')->where('stk_id',$v['stk_id'])->get();
                        $stock = current($stock->toArray());
                        $stock_data = array();
                        $stock_data['stk_lock_num'] = $stock->stk_lock_num - $v->dpd_num;
                        if($stock_data['stk_lock_num'] < 0){
                            $stock_data['stk_lock_num'] = 0;
                        }
                        $db->table('stock')
                            ->where('stk_id',$v->stk_id)
                            ->update($stock_data);

                        $db->table('dpdetail')
                            ->delete($v->dpd_id);

                        $si_assigned_num = $soItem['si_assigned_num'] - $v->dpd_num;

                        app(RhawnSoitemsRepository::class)->update(['si_patch_status'=>'0','si_assigned_num'=>$si_assigned_num],$si_id);
                    }
                }
            }
        }
        $customer = $db->table('customers')
            ->where('cus_id',$sOrderDetail['cus_id'])
            ->first();
        //判断是否是最后一条非运费明细
        $note = $sOrderDetail['so_no'].'取消明细：'.$soItem['p_code'];
        if($siAmountTotalPid > 0){
            $refund_amount = $soItem['si_amount'];

            if($sOrderDetail['so_pay_status'] > 0){
                // 退款
                if($refundInfo->transfer_pre == 0){

                    $refund_data = [];
                    $refund_data['r_company'] = $customer->cus_company;
                    $refund_data['r_amount'] = $refund_amount;
                    $refund_data['r_ctime'] = time();
                    $refund_data['r_note'] = $note;
                    $refund_data['r_bank_account'] = '';
                    $refund_data['r_confirm_note'] = '';
                    $db->table('refund')->insertGetId($refund_data);

                }else{

                    $prepaylog_data = [];
                    $prepaylog_data['cus_id'] = $sOrderDetail['cus_id'];
                    $prepaylog_data['prepaylog_amount'] = $refund_amount;
                    $prepaylog_data['prepaylog_note'] = $note;
                    $prepaylog_data['prepaylog_date'] = time();
                    $prepaylog_data['prepaylog_time'] = time();
                    $prepaylog_data['prepaylog_operator'] = auth()->user()->id;
                    $prepaylog_data['prepaylog_unpay'] = 0;
                    $prepaylog_data['prepaylog_if_abnormal'] = 0;
                    $prepaylog_data['prepaylog_if_deposit'] = 0;
                    $prepaylog_data['prepaylog_if_deal'] = 0;

                    $prepayId = $db->table('prepaylog')->insertGetId($prepaylog_data);

                    $sum = $db->table('prepaylog')
                        ->where('cus_id',$sOrderDetail['cus_id'])
                        ->sum('prepaylog_amount');

                    $db->table('customers')
                        ->where('cus_id',$sOrderDetail['cus_id'])
                        ->update(['cus_prepay' => $sum]);

                    $si_cancel_time = time();
                }
            }

            app(RhawnSordersRepository::class)->update(['so_total'=>$siAmountTotal],$sOrderItem['so_id']);
            app(RhawnSoitemsRepository::class)->update(['si_if_cancel'=>'1','si_cancel_time'=>$si_cancel_time],$si_id);

        }else {

            if($sOrderDetail['so_pay_status'] > 0){
                //得到退款金额
                $sOrder = app(RhawnSordersRepository::class)->find($sOrderItem['so_id']);
                if (!$sOrder) {
                    throw new \LogicException('数据不存在！', 500);
                }
                $sOrder = $sOrder->toArray();

                $refund_amount = $sOrder['so_total'];

                if($refundInfo->transfer_pre == 0){

                    $refund_data = array();
                    $refund_data['r_company'] = $customer->cus_company;
                    $refund_data['r_amount'] = $refund_amount;
                    $refund_data['r_ctime'] = time();
                    $refund_data['r_note'] = $note;
                    $refund_data['r_bank_account'] = '';
                    $refund_data['r_confirm_note'] = '';
                    $db->table('refund')->insertGetId($refund_data);

                }else{

                    $prepaylog_data = [];
                    $prepaylog_data['cus_id'] = $sOrderDetail['cus_id'];
                    $prepaylog_data['prepaylog_amount'] = $refund_amount;
                    $prepaylog_data['prepaylog_note'] = $note;
                    $prepaylog_data['prepaylog_date'] = time();
                    $prepaylog_data['prepaylog_time'] = time();
                    $prepaylog_data['prepaylog_operator'] = auth()->user()->id;
                    $prepaylog_data['prepaylog_unpay'] = 0;
                    $prepaylog_data['prepaylog_if_abnormal'] = 0;
                    $prepaylog_data['prepaylog_if_deposit'] = 0;
                    $prepaylog_data['prepaylog_if_deal'] = 0;

                    $prepayId = $db->table('prepaylog')->insertGetId($prepaylog_data);

                    $sum = $db->table('prepaylog')
                        ->where('cus_id', $sOrderDetail['cus_id'])
                        ->sum('prepaylog_amount');

                    $db->table('customers')
                        ->where('cus_id', $sOrderDetail['cus_id'])
                        ->update(['cus_prepay' => $sum]);

                    $si_cancel_time = time();
                }
            }

            app(RhawnSordersRepository::class)->update(['so_review_status' => '3', 'so_total' => '0'], $sOrderItem['so_id']);
            app(RhawnSoitemsRepository::class)->update(['si_if_cancel' => '1', 'si_cancel_time' => $si_cancel_time], $si_id);
        }

        $db->table('logs')->insertGetId(
            ['l_obj'=>$sOrderItem['so_id'],'l_type'=>'sorders_mng','l_op'=>'cancelsi','l_op_name'=>'取消明细','l_note'=>$note,'l_timestamp'=>time(),'u_id'=>auth()->user()->id]
        );

        return true;
    }

    public function getRhawnSorderTaskLog($request)
    {

        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');
        if(!isset($params['type']) || empty($params['type'])){
            $params['type'] = 'rhaw_nsorder_refund';
        }

        $taskService = app(ToolsTaskRepository::class);
        $taskLogsList = $taskService->getTaskLogsFromType($params['type'],$params['task_id']);

        return $taskLogsList;
    }

}
