<?php

namespace App\Support\Traits;

use App\Repositories\Contracts\MenusRepository;
use App\Repositories\Models\AdminUsers;
use App\Services\RoleAndPermissionsService;

use App\Repositories\Models\Menus;
use Illuminate\Support\Str;
use Spatie\Permission\Models\Permission;

trait InitMenusAndPermission
{
    protected $menusRepository = null;

    public function __construct(MenusRepository $menusRepository)
    {
       $this->menusRepository = $menusRepository;
    }

    protected function initMenu($menus)
    {
        $menuModel = app(Menus::class);

        foreach($menus as $menu){
            $parentMenu = $menuModel->where('menu_name',$menu['parent_id'])->first();
            if(!$parentMenu){
                throw new \LogicException('上级菜单不存在！',500);
            }

            if('button' == $menu['menu_type']){
                $permission = Permission::query()->where(['menu_id' => 0, 'sys_default' => 1,'guard_name' => config('auth.defaults.guard'), 'action' => $menu['action']])->first();
                if(!$permission){
                    throw new \LogicException('按钮动作不存在！',500);
                }
            }

            $filterParams = [];
            collect($this->menusRepository->rules['create'])->map(function($rule,$key) use ($menu, &$filterParams){
                if(isset($menu[$key]) && !empty($menu[$key])){
                    $filterParams[$key] = $menu[$key];
                } else{
                    $filterParams[$key] = '';
                }
            });
            $filterParams['parent_id'] = $parentMenu['id'];
            $filterParams['menu_icon'] = $menu['menu_icon'];
            $filterParams['component'] = $menu['component'];
            $filterParams['created_by'] = 1;
            $filterParams['status'] = intval($menu['status']);

            $result = $this->menusRepository->create($filterParams);
            if($result){
                $permission = [];
                $permission['name'] = $result['menu_name'];
                $permission['menu_id'] = $result['id'];
                if(isset($menu['menu_type']) && !empty($menu['menu_type']) && 'button' == $menu['menu_type']){
                    $permission['action'] = $menu['menu_name'];
                    $permission['permission_type'] = 'button';
                }else{
                    $permission['permission_type'] = 'menu';
                    $permission['action'] = $menu['menu_name'].'_list';
                }
                $permission['sys_default'] = 0;
                $permission['remark'] = $result['menu_name'].'的相关权限';
                $permission['guard_name'] = config('auth.defaults.guard');
                //增加对应的按钮权限
                $permissionResult = Permission::create($permission);
                if($permissionResult){
                    $user = app(AdminUsers::class)->where('id',1)->get()->first();
                    app(RoleAndPermissionsService::class)->assignRoleAndPermissionToUser($permissionResult, $user);
                }
            }
        }
    }
}
